/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 *
 * Copyright © 2007 Gerd Kohlberger <lowfi@chello.at>
 */

#include <gtk/gtk.h>
#include <glade/glade.h>

#include "mt-main.h"
#include "mt-preferences.h"
#include "mt-ct-window.h"

extern GladeXML *xml;

static void
update_mode_sensitivity (DwellMode mode)
{
    GtkWidget *w;

    w = glade_xml_get_widget (xml, "box_ctw");
    gtk_widget_set_sensitive (w, mode);

    w = glade_xml_get_widget (xml, "box_gesture");
    gtk_widget_set_sensitive (w, !mode);
}

static gboolean
verfiy_gesture_setting (MTClosure *mt, gint value, DwellClickType type)
{
    gint i;

    for (i = 0; i < N_CLICK_TYPES; i++) {
	if (i == type)
	    continue;

	if (mt->dwell_dirs[i] == value)
	    return FALSE;
    }

    return TRUE;
}

static void
preferences_closed_cb (GtkButton *button, gpointer data)
{
    gtk_widget_hide (glade_xml_get_widget (xml, "preferences"));
}

static void
threshold_changed_cb (GtkRange *range, gpointer data)
{
    MTClosure *mt = (MTClosure *) data;

    mt->threshold = gtk_range_get_value (range);
    gconf_client_set_int (mt->client,
			  MT_GCONF_HOME "/threshold",
			  mt->threshold,
			  NULL);
}

static void
delay_enabled_cb (GtkToggleButton *button, gpointer data)
{
    MTClosure *mt = (MTClosure *) data;
    gboolean active;
    gint ret;

    active = gtk_toggle_button_get_active (button);

    if (active && mt->dwell_enabled) {
	ret = mt_show_dialog (_("Enable Delay Click"),
			      _("Dwell Click is currently active and will "
			      "be disabled."),
			       GTK_MESSAGE_QUESTION);

	if (ret == GTK_RESPONSE_NO ||
	    ret == GTK_RESPONSE_DELETE_EVENT) {
	    gtk_toggle_button_set_active (button, FALSE);
	    return;
	}
	else {
	    GtkWidget *w;

	    w = glade_xml_get_widget (xml, "dwell_enable");
	    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w), FALSE);
	}
    }

    mt->delay_enabled = active;
    gconf_client_set_bool (mt->client,
			   MT_GCONF_HOME "/delay_enabled",
			   mt->delay_enabled,
			   NULL);
}

static void
delay_time_cb (GtkRange *range, gpointer data)
{
    MTClosure *mt = (MTClosure *) data;

    mt->delay_time = gtk_range_get_value (range);
    gconf_client_set_float (mt->client,
			    MT_GCONF_HOME "/delay_time",
			    mt->delay_time,
			    NULL);
}

static void
dwell_enabled_cb (GtkToggleButton *button, gpointer data)
{
    MTClosure *mt = (MTClosure *) data;
    gboolean active;
    gint ret;

    active = gtk_toggle_button_get_active (button);

    if (active && mt->delay_enabled) {
	ret = mt_show_dialog (_("Enable Dwell Click"),
			      _("Delay Click is currently active and will "
			      "be disabled."),
			      GTK_MESSAGE_QUESTION);

	if (ret == GTK_RESPONSE_NO ||
	    ret == GTK_RESPONSE_DELETE_EVENT) {
	    gtk_toggle_button_set_active (button, FALSE);
	    return;
	}
	else {
	    GtkWidget *w;

	    w = glade_xml_get_widget (xml, "delay_enable");
	    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w), FALSE);
	}
    }

    mt->dwell_enabled = active;
    gconf_client_set_bool (mt->client,
			   MT_GCONF_HOME "/dwell_enabled",
			   mt->dwell_enabled,
			   NULL);

    mt_ctw_update_sensitivity (mt);
}

static void
dwell_time_cb (GtkRange *range, gpointer data)
{
    MTClosure *mt = (MTClosure *) data;

    mt->dwell_time = gtk_range_get_value (range);
    gconf_client_set_float (mt->client,
			    MT_GCONF_HOME "/dwell_time",
			    mt->dwell_time,
			    NULL);
}

static void
dwell_show_ctw_cb (GtkToggleButton *button, gpointer data)
{
    MTClosure *mt = (MTClosure *) data;
    GtkWidget *ctw;

    ctw = glade_xml_get_widget (xml, "click-type");
    mt->dwell_show_ctw = gtk_toggle_button_get_active (button);
    gconf_client_set_bool (mt->client,
			   MT_GCONF_HOME "/dwell_show_ctw",
			   mt->dwell_show_ctw,
			   NULL);

    if (mt->dwell_show_ctw)
	gtk_widget_show (ctw);
    else
	gtk_widget_hide (ctw);
}

static void
dwell_mode_cb (GtkToggleButton *button, gpointer data)
{
    MTClosure *mt = (MTClosure *) data;
    GSList *group;

    if (!gtk_toggle_button_get_active (button))
	return;

    group = gtk_radio_button_get_group (GTK_RADIO_BUTTON(button));
    mt->dwell_mode = g_slist_index (group, (gconstpointer) button);
    gconf_client_set_int (mt->client,
			  MT_GCONF_HOME "/dwell_mode",
			  (gint) mt->dwell_mode,
			  NULL);

    update_mode_sensitivity (mt->dwell_mode);
    mt_ctw_update_sensitivity (mt);
}

static void
dwell_gesture_single_cb (GtkComboBox *combo, gpointer data)
{
    MTClosure *mt = (MTClosure *) data;
    gint active;

    active = gtk_combo_box_get_active (combo);

    if (!verfiy_gesture_setting (mt, active, DWELL_CLICK_TYPE_SINGLE)) {
	mt->dwell_dirs[DWELL_CLICK_TYPE_SINGLE] = DIRECTION_DISABLE;
	gtk_combo_box_set_active (combo, DIRECTION_DISABLE);
    }
    else
	mt->dwell_dirs[DWELL_CLICK_TYPE_SINGLE] = active;

    gconf_client_set_int (mt->client,
			  MT_GCONF_HOME "/dwell_gesture_single",
			  mt->dwell_dirs[DWELL_CLICK_TYPE_SINGLE],
			  NULL);
}

static void
dwell_gesture_double_cb (GtkComboBox *combo, gpointer data)
{
    MTClosure *mt = (MTClosure *) data;
    gint active;

    active = gtk_combo_box_get_active (combo);

    if (!verfiy_gesture_setting (mt, active, DWELL_CLICK_TYPE_DOUBLE)) {
	mt->dwell_dirs[DWELL_CLICK_TYPE_DOUBLE] = DIRECTION_DISABLE;
	gtk_combo_box_set_active (combo, DIRECTION_DISABLE);
    }
    else
	mt->dwell_dirs[DWELL_CLICK_TYPE_DOUBLE] = active;

    gconf_client_set_int (mt->client,
			  MT_GCONF_HOME "/dwell_gesture_double",
			  mt->dwell_dirs[DWELL_CLICK_TYPE_DOUBLE],
			  NULL);
}

static void
dwell_gesture_drag_cb (GtkComboBox *combo, gpointer data)
{
    MTClosure *mt = (MTClosure *) data;
    gint active;

    active = gtk_combo_box_get_active (combo);

    if (!verfiy_gesture_setting (mt, active, DWELL_CLICK_TYPE_DRAG)) {
	mt->dwell_dirs[DWELL_CLICK_TYPE_DRAG] = DIRECTION_DISABLE;
	gtk_combo_box_set_active (combo, DIRECTION_DISABLE);
    }
    else
	mt->dwell_dirs[DWELL_CLICK_TYPE_DRAG] = active;

    gconf_client_set_int (mt->client,
			  MT_GCONF_HOME "/dwell_gesture_drag",
			  mt->dwell_dirs[DWELL_CLICK_TYPE_DRAG],
			  NULL);
}

static void
dwell_gesture_right_cb (GtkComboBox *combo, gpointer data)
{
    MTClosure *mt = (MTClosure *) data;
    gint active;

    active = gtk_combo_box_get_active (combo);

    if (!verfiy_gesture_setting (mt, active, DWELL_CLICK_TYPE_RIGHT)) {
	mt->dwell_dirs[DWELL_CLICK_TYPE_RIGHT] = DIRECTION_DISABLE;
	gtk_combo_box_set_active (combo, DIRECTION_DISABLE);
    }
    else
	mt->dwell_dirs[DWELL_CLICK_TYPE_RIGHT] = active;

    gconf_client_set_int (mt->client,
			  MT_GCONF_HOME "/dwell_gesture_right",
			  mt->dwell_dirs[DWELL_CLICK_TYPE_RIGHT],
			  NULL);
}

void
mt_preferences_init (MTClosure *mt)
{
    GtkWidget *w;

    w = glade_xml_get_widget (xml, "preferences");
    g_signal_connect (G_OBJECT(w), "delete-event", 
		      G_CALLBACK(gtk_widget_hide_on_delete), NULL);

    w = glade_xml_get_widget (xml, "close");
    g_signal_connect (G_OBJECT(w), "clicked", 
		      G_CALLBACK(preferences_closed_cb), NULL);

    /* general tab */
    w = glade_xml_get_widget (xml, "threshold");
    gtk_range_set_value (GTK_RANGE(w), mt->threshold);
    g_signal_connect (G_OBJECT(w), "value-changed", 
		      G_CALLBACK(threshold_changed_cb), (gpointer) mt);

    /* delay tab */
    w = glade_xml_get_widget (xml, "delay_enable");
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w), mt->delay_enabled);
    g_signal_connect (G_OBJECT(w), "toggled", 
		      G_CALLBACK(delay_enabled_cb), (gpointer) mt);

    w = glade_xml_get_widget (xml, "delay_time");
    gtk_range_set_value (GTK_RANGE(w), mt->delay_time);
    g_signal_connect (G_OBJECT(w), "value-changed", 
		      G_CALLBACK(delay_time_cb), (gpointer) mt);

    /* dwell tab */
    w = glade_xml_get_widget (xml, "dwell_enable");
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w), mt->dwell_enabled);
    g_signal_connect (G_OBJECT(w), "toggled", 
		      G_CALLBACK(dwell_enabled_cb), (gpointer) mt);

    w = glade_xml_get_widget (xml, "dwell_time");
    gtk_range_set_value (GTK_RANGE(w), mt->dwell_time);
    g_signal_connect (G_OBJECT(w), "value-changed", 
		      G_CALLBACK(dwell_time_cb), (gpointer) mt);

    w = glade_xml_get_widget (xml, "dwell_show_ctw");
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w), mt->dwell_show_ctw);
    g_signal_connect (G_OBJECT(w), "toggled", 
		      G_CALLBACK(dwell_show_ctw_cb), (gpointer) mt);

    w = glade_xml_get_widget (xml, "dwell_mode_ctw");
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w), mt->dwell_mode);
    g_signal_connect (G_OBJECT(w), "toggled", 
		      G_CALLBACK(dwell_mode_cb), (gpointer) mt);

    w = glade_xml_get_widget (xml, "dwell_mode_gesture");
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w), !mt->dwell_mode);
    g_signal_connect (G_OBJECT(w), "toggled", 
		      G_CALLBACK(dwell_mode_cb), (gpointer) mt);

    update_mode_sensitivity (mt->dwell_mode);

    w = glade_xml_get_widget (xml, "dwell_gest_single");
    gtk_combo_box_set_active (GTK_COMBO_BOX(w),
			      mt->dwell_dirs[DWELL_CLICK_TYPE_SINGLE]);
    g_signal_connect (G_OBJECT(w), "changed", 
		      G_CALLBACK(dwell_gesture_single_cb), (gpointer) mt);

    w = glade_xml_get_widget (xml, "dwell_gest_double");
    gtk_combo_box_set_active (GTK_COMBO_BOX(w),
			      mt->dwell_dirs[DWELL_CLICK_TYPE_DOUBLE]);
    g_signal_connect (G_OBJECT(w), "changed", 
		      G_CALLBACK(dwell_gesture_double_cb), (gpointer) mt);

    w = glade_xml_get_widget (xml, "dwell_gest_drag");
    gtk_combo_box_set_active (GTK_COMBO_BOX(w),
			      mt->dwell_dirs[DWELL_CLICK_TYPE_DRAG]);
    g_signal_connect (G_OBJECT(w), "changed", 
		      G_CALLBACK(dwell_gesture_drag_cb), (gpointer) mt);

    w = glade_xml_get_widget (xml, "dwell_gest_right");
    gtk_combo_box_set_active (GTK_COMBO_BOX(w),
			      mt->dwell_dirs[DWELL_CLICK_TYPE_RIGHT]);
    g_signal_connect (G_OBJECT(w), "changed", 
		      G_CALLBACK(dwell_gesture_right_cb), (gpointer) mt);
}
