/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 *
 * Copyright © 2007 Gerd Kohlberger <lowfi@chello.at>
 */

#include <gtk/gtk.h>
#include <glade/glade.h>

#include "mt-main.h"
#include "mt-trayicon.h"

extern GladeXML *xml;

static void
tray_popup_cb (GtkStatusIcon *icon, guint button, guint time, gpointer data) 
{
    gtk_menu_popup (GTK_MENU(data), 0, 0, 0, 0, button, time);
}

static void
tray_activate_cb (GtkStatusIcon *icon, gpointer data) 
{
    mt_show_preferences ();
}

static void
preferences_dialog_cb (GtkMenuItem *item, gpointer data)
{
    mt_show_preferences ();
}

static void
about_response (GtkAboutDialog *dialog, gint response, gpointer data)
{
    gtk_widget_hide (GTK_WIDGET(dialog));
}

static void
about_dialog_cb (GtkMenuItem *item, gpointer data)
{
    GtkWidget *w;

    w = glade_xml_get_widget (xml, "about");

    if (GTK_WIDGET_VISIBLE(w))
	gtk_window_present (GTK_WINDOW(w));
    else
	gtk_widget_show (w);
}

static void
quit_cb (GtkMenuItem *item, gpointer data)
{
    spi_shutdown ();
}

void
mt_trayicon_init (MTClosure *mt)
{
    GtkStatusIcon *icon;
    GtkWidget *menu, *item, *w;

    menu = gtk_menu_new ();

    icon = gtk_status_icon_new_from_icon_name (MT_ICON_NAME);
    gtk_status_icon_set_tooltip (icon, "MouseTweaks");

    g_signal_connect (G_OBJECT(icon), "popup-menu",
		      G_CALLBACK(tray_popup_cb), (gpointer) menu);
    g_signal_connect (G_OBJECT(icon), "activate",
		      G_CALLBACK(tray_activate_cb), NULL);

    item = gtk_image_menu_item_new_from_stock (GTK_STOCK_PREFERENCES, NULL);
    g_signal_connect (G_OBJECT(item), "activate",
		      G_CALLBACK(preferences_dialog_cb), NULL);
    gtk_menu_shell_append (GTK_MENU_SHELL(menu), item);
    gtk_widget_show (item);

    item = gtk_image_menu_item_new_from_stock (GTK_STOCK_ABOUT, NULL);
    g_signal_connect (G_OBJECT(item), "activate",
		      G_CALLBACK(about_dialog_cb), (gpointer) mt);
    gtk_menu_shell_append (GTK_MENU_SHELL(menu), item);
    gtk_widget_show (item);

    item = gtk_separator_menu_item_new ();
    gtk_menu_shell_append (GTK_MENU_SHELL(menu), item);
    gtk_widget_show (item);

    item = gtk_image_menu_item_new_from_stock (GTK_STOCK_QUIT, NULL);
    g_signal_connect (G_OBJECT(item), "activate",
		      G_CALLBACK(quit_cb), NULL);
    gtk_menu_shell_append (GTK_MENU_SHELL(menu), item);
    gtk_widget_show (item);

    /* about dialog */
    gtk_window_set_default_icon_name (MT_ICON_NAME);

    w = glade_xml_get_widget (xml, "about");
    g_object_set (w, "version", VERSION, NULL);

    g_signal_connect (G_OBJECT(w), "delete-event",
		      G_CALLBACK(gtk_widget_hide_on_delete), NULL);
    g_signal_connect (G_OBJECT(w), "response",
		      G_CALLBACK(about_response), NULL);
}
